/**
 ****************************************************************************
 *
 *  @file   clientLGV.cpp
 *  @author Ole-Ivar Holthe
 *  @date   2008-04-01
 *  @brief  Implements the CGXClientLGV class.
 *  @update 02/04/08: Added ifdefs for Linux compilation (LINUX_COMPILE)
 *
 *  Copyright � Gridmedia Technologies AS 1996-2008. All Rights Reserved.
 *
 ****************************************************************************/

#include "clientLGV.h"

#ifndef GSTRACE
	void GSTRACE1(const char* fmt, ...) {}
	#define GSTRACE GSTRACE1
#endif // GSTRACE

/**
 ****************************************************************************
 * 
 *  @class  CGXClientLGV
 *  @brief  The livegames sharing class.
 *
 ****************************************************************************/

CGXClientLGV::CGXClientLGV()
{
    m_bConnected = FALSE;
    memset(&m_addrServer, 0, sizeof(struct sockaddr_in));

    m_szLoginName[0] = 0;
    m_szLoginPassword[0] = 0;
    m_nLoginGameID = -1;
    m_nEchoClock = 0;
    m_nEchoLatency = 0;
    m_nSendInputSeqNo = 0;

    m_nPacketSize = 8192;
    m_nVidCodec = CODEC_ID_WMV2;
    m_nVidBitrate = 0;
    m_nVidFmtDX = 0;
    m_nVidWidth = 0;
    m_nVidHeight = 0;
    m_nVidFps = 0;
    m_nAudCodec = CODEC_ID_WMAV2;
    m_nAudBitrate = 0;
    m_nAudChannels = 0;
    m_nAudSampleRate = 0;
    m_nAudBitsPerSample = 0;
    m_nInpKBLayout = 0;

#ifdef LINUX_COMPILE
	gettimeofday(&start,NULL);
#endif
}

CGXClientLGV::~CGXClientLGV()
{
    // Cleanup socket
    m_sock.Destroy();
}


/*
 ****************************************************************************
 *
 *  Protocol
 *
 ****************************************************************************/

/**
 *  Sends CONNECT VIDEO.
 */
BOOL CGXClientLGV::DoSendConnectVideo(LPCTSTR szServerIP, int nServerPort)
{
    // State
    m_bConnected = FALSE;

    // Create socket
    if (!m_sock.Create())
        return FALSE;

    // Set socket rcvbuf size
    // Note: Without this, there will be a lot of packetloss.
    unsigned int optval = 65000;
    setsockopt(m_sock.m_sd, SOL_SOCKET, SO_RCVBUF, reinterpret_cast<char*>(&optval), sizeof(optval));

#ifdef LINUX_COMPILE
    // Why bind?
#else
    // Bind socket
    // Note: Port 0 enables local client and server
    if (!m_sock.Bind(NULL, 0 /*port-all*/)) {
        m_sock.Destroy();
        return FALSE;
    }
#endif // LINUX_COMPILE

    // Resolve server address
    if (!CGSSockUDP::Resolve(&m_addrServer, szServerIP, nServerPort /*13580*/)) {
        m_sock.Destroy();
        return FALSE;
    }

    // Connect message
#ifdef LINUX_COMPILE
    char str[256];
    bzero(str,256);
    sprintf(str,
	"CONNECT VIDEO\nVERSION 5.0.0.0\n%s\n%s\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n\n",
		m_szLoginName,
		m_szLoginPassword,
		m_nLoginGameID,
		m_nPacketSize,
		m_nVidCodec,
		m_nVidBitrate,
		m_nVidFmtDX,
		m_nVidWidth,
		m_nVidHeight,
		m_nVidFps,
		m_nAudCodec,
		m_nAudBitrate,
		m_nAudChannels,
		m_nAudSampleRate,
		m_nAudBitsPerSample,
		m_nInpKBLayout
    );

    	// Send msg
	int bytes = sendto(m_sock.m_sd, (const char*)str, strlen(str)+1, 0, 
                       (struct sockaddr*)&m_addrServer, sizeof(m_addrServer));

#else
    CGSString str;
    char buf[256];
    str = "CONNECT VIDEO\nVERSION 5.0.0.0\n";
    str += m_szLoginName;  str += "\n";
    str += m_szLoginPassword;  str += "\n";
    str += itoa(m_nLoginGameID, buf, 10);  str += "\n";
    str += itoa(m_nPacketSize, buf, 10);  str += "\n";
    str += itoa(m_nVidCodec, buf, 10);  str += "\n";
    str += itoa(m_nVidBitrate, buf, 10);  str += "\n";
    str += itoa(m_nVidFmtDX, buf, 10);  str += "\n";
    str += itoa(m_nVidWidth, buf, 10);  str += "\n";
    str += itoa(m_nVidHeight, buf, 10);  str += "\n";
    str += itoa(m_nVidFps, buf, 10);  str += "\n";
    str += itoa(m_nAudCodec, buf, 10);  str += "\n";
    str += itoa(m_nAudBitrate, buf, 10);  str += "\n";
    str += itoa(m_nAudChannels, buf, 10);  str += "\n";
    str += itoa(m_nAudSampleRate, buf, 10);  str += "\n";
    str += itoa(m_nAudBitsPerSample, buf, 10);  str += "\n";
    str += itoa(m_nInpKBLayout, buf, 10);  str += "\n";
    str += "\n\0";

    // Send msg
	int addrLen = sizeof(struct sockaddr_in);
	int bytes = sendto(m_sock.m_sd, (const char*)str, (int)str.GetLength()+1, 0, 
                       (struct sockaddr*)&m_addrServer, addrLen);
#endif // LINUX_COMPILE

    if (bytes == -1) {

		// Error transmitting data
        m_sock.Destroy();
		return FALSE;
	}

    m_nEchoClock = clock(); // NEW
    return TRUE;
}

/**
 *  Checks the socket for any incoming data.
 */
ULONG CGXClientLGV::DoListen()
{
    // Get amount of bytes pending on the socket input buffer
#ifdef LINUX_COMPILE
	unsigned long bytes = 0;
   	if ( ioctl(m_sock.m_sd, FIONREAD, &bytes) == -1 )
		GSTRACE("CGXClientLGV::DoListen(): Failed to query socket buffer\n");
#else
   u_long bytes = 0;
    if (ioctlsocket(m_sock.m_sd, FIONREAD , &bytes) == SOCKET_ERROR) {
        GSTRACE("CGXClientLGV::DoListen(): "
                "Failed to query socket buffer: %d\n", WSAGetLastError());
    }
#endif // LINUX_COMPILE

    return bytes;
}

BOOL CGXClientLGV::OnConnect()
{
    // Receive bytes from server
#ifdef LINUX_COMPILE
    int bytes = recvfrom(m_sock.m_sd, m_pktbuf, sizeof(m_pktbuf), 0, NULL, NULL); // TODO: Ugly hack
    if ((bytes < 0) || (m_pktbuf[bytes-1] != 0))
    {
	GSTRACE("CGXClientLGV::OnConnect(): Failed to receive datagram");
#else

    struct sockaddr_in addrFrom;
    int cs = (int)sizeof(struct sockaddr_in);
    int bytes = recvfrom(m_sock.m_sd, m_pktbuf, sizeof(m_pktbuf), 0, 
                         (struct sockaddr*)&addrFrom, &cs);
    if ((bytes < 0) || (m_pktbuf[bytes-1] != 0)) {
        GSTRACE("CGXClientLGV::OnConnect(): "
                "Failed to receive datagram: %d\n", WSAGetLastError());
#endif // LINUX_COMPILE
        m_sock.Destroy();
        return FALSE;
    }

    // Print packet (debug)
#ifdef LINUX_COMPILE
    GSTRACE("CGXClientLGV::OnConnect(): Incoming packet of %d bytes:\n%s\n\n", 
        bytes, m_pktbuf);
#else
    GSTRACE("CGXClientLGV::OnConnect(): Incoming packet of %d bytes from %u.%u.%u.%u:\n%s\n\n", 
        bytes, PRNIP(addrFrom), m_pktbuf);
#endif // LINUX_COMPILE

    // Parse packet
    char* ppsz[128];
    char* psz1 = (char*)m_pktbuf;
    int ci;
    for (ci = 0; ci < 128; ci++) {
        ppsz[ci] = psz1;
        psz1 = strchr(psz1, '\n');
        if (psz1 == 0) break;
        *psz1 = 0; psz1++;
    }

    if ((ci < 15) || (strcmp("OK", ppsz[0]) != 0)) {
        GSTRACE("CGXClientLGV::OnConnect(): Illegal protocol data\n");
        m_sock.Destroy();
        return FALSE;
    }

    // Get params
    m_nPacketSize =       atoi(ppsz[1]);
    m_nVidCodec =         atoi(ppsz[2]);
    m_nVidBitrate =       atoi(ppsz[3]);
    m_nVidFmtDX =         atoi(ppsz[4]);
    m_nVidWidth =         atoi(ppsz[5]);
    m_nVidHeight =        atoi(ppsz[6]);
    m_nVidFps =           atoi(ppsz[7]);
    m_nAudCodec =         atoi(ppsz[8]);
    m_nAudBitrate =       atoi(ppsz[9]);
    m_nAudChannels =      atoi(ppsz[10]);
    m_nAudSampleRate =    atoi(ppsz[11]);
    m_nAudBitsPerSample = atoi(ppsz[12]);
    m_nInpKBLayout =      atoi(ppsz[13]);

    // Clear cache
    m_cache.Clear();
 
    m_bConnected = TRUE;

    return TRUE;
}

/**
 *  Sends an input device packet to server.
 *
 *  Keyboard (nDevice == 1):
 *  ------------------------
 *      nVirtKey = Virtual key code (ref Win32)
 *      nData1 = 0x80 (down) | 0x00 (up)
 *
 *  Mouse (nDevice == 2):
 *  ---------------------
 *      nVirtKey =
 *          0x00 (XY) | 0x01 (Z) |
 *          0x80 (B0) |..| 0x87 (B7)
 *      nData1 = dx | dz | (0x80|0x00)
 *      nData2 = dy
 *
 *  Echo (nDevice == 0):
 *  --------------------
 *      nVirtKey = 0x80 (echo back) | 0x00
 *      nData1+(nData2<<16) = clock()
 */
BOOL CGXClientLGV::DoSendInput(short nDevice, short nVirtKey, short nData1, short nData2)
{

    // Make packet
    BYTE pktbuf[16];
    memset(pktbuf, 0, sizeof(pktbuf));
    *((BYTE*)(pktbuf + 0)) = 'I';

    *((short*)(pktbuf + 1)) = nDevice;
    *((short*)(pktbuf + 3)) = nVirtKey;
    *((short*)(pktbuf + 5)) = nData1;
    *((short*)(pktbuf + 7)) = nData2;
    *((UINT*)(pktbuf + 9)) = m_nSendInputSeqNo++;

    int len = sizeof(pktbuf);
    int cs = (int)sizeof(struct sockaddr_in);
    int bytes = sendto(m_sock.m_sd, (char*)pktbuf, len, 0, (struct sockaddr*)&m_addrServer, cs);

#ifndef LINUX_COMPILE
    if (bytes != len) {
        GSTRACE("CGXClientLGV::DoSendInput(): Failed to send to %u.%u.%u.%u: %d\n", 
            PRNIP(m_addrServer), WSAGetLastError());
        return FALSE;
	}
#else
//printf("[ClientLGV] DoSendInput: device=%i vk=%i d1=%i d2=%i\n",nDevice, nVirtKey, nData1, nData2);
//printf("[ClientLGV] DoSendInput packet of %i bytes: ",len);

    if (bytes != len)
    {
	GSTRACE("CGXClientLGV::DoSendInput(): Failed to send input\n"); 
	return FALSE;
    }
#endif // LINUX_COMPILE

    return TRUE;
}

BOOL CGXClientLGV::OnVidPkt()
{
#ifdef LINUX_COMPILE
    int bytes = recvfrom(m_sock.m_sd, (char*)m_pktbuf,sizeof(m_pktbuf), 0, NULL, NULL);
#else
    struct sockaddr_in addrFrom;
    int cs = (int)sizeof(struct sockaddr_in);
    int bytes = recvfrom(m_sock.m_sd, (char*)m_pktbuf, sizeof(m_pktbuf), 0, 
                         (struct sockaddr*)&addrFrom, &cs);
#endif // LINUX_COMPILE

#ifdef LINUX_COMPILE
    #ifndef MAKELONG
	typedef unsigned short WORD;
	typedef unsigned long DWORD;
	#define MAKELONG(low, high)   ((DWORD)(((WORD)(low)) | (((DWORD)((WORD)(high))) << 16)))
    #endif
#endif

    if ((bytes == 16) && (m_pktbuf[0] == 'I')) {
        short nData1 = *((short*)(m_pktbuf + 5));
        short nData2 = *((short*)(m_pktbuf + 7));
#ifdef LINUX_COMPILE
	unsigned long t0 = MAKELONG(nData1, nData2);
	unsigned long t1 = GetProcesstimeMS();
#else
        clock_t t0 = MAKELONG(nData1, nData2);
	clock_t t1 = clock();
#endif
        m_nEchoLatency = t1 - t0;
	return OnEchoBack();
    }

    if (bytes < 26) {
        GSTRACE("CGXClientLGV::OnVidPkt(): "
                "Failed to receive datagram: %d\n", 
#ifdef LINUX_COMPILE
	0
#else
	WSAGetLastError()
#endif // LINUX_COMPILE
	);
        return FALSE;
    }


    // Read packet header
    BYTE nType = *((BYTE*)(m_pktbuf + 0));
    LONGLONG nTimestamp = *((LONGLONG*)(m_pktbuf + 1));
    int nFlags = *((int*)(m_pktbuf + 9));
    int nPktNum = *((int*)(m_pktbuf + 13));
    int nPktCount = *((int*)(m_pktbuf + 17));
    UINT nPktSeqNo = *((UINT*)(m_pktbuf + 21));

    // Trace
    /*GSTRACE("pktrcv:  type=%c  ts=%d  fl=%d  no=%d  size=%d  count=%d\n", 
            nType, (long)nTimestamp, nFlags, nPktNum, bytes, nPktCount);*/

    // Some sanity checks
    if ((nType != 'A' && nType != 'a' && nType != 'V' && nType != 'v') ||
        (nPktNum < 0) || (nPktNum > nPktCount) ||
        (nPktCount <= 0) || (nPktCount > 1024))
    {
        GSTRACE("CGXClientLGV::OnVidPkt(): "
                "Bad pkt:  type=%c  ts=%d  fl=%d  no=%d  size=%d  count=%d  seqno=%d\n",
                nType, (long)nTimestamp, nFlags, nPktNum, bytes, nPktCount, nPktSeqNo);
        return FALSE;
    }
    // Put in cache
    BOOL r = m_cache.Put(nType, nTimestamp, nFlags, 
                         nPktNum, nPktCount, nPktSeqNo,
                         (BYTE*)(m_pktbuf+25), bytes-25);
    if (!r) {
        GSTRACE("CGXClientLGV::OnVidPkt(): Out of cache memory!\n");
        return FALSE;
    }

    // NEW: Echo (every 2 secs)
#ifdef LINUX_COMPILE
    if ((GetProcesstimeMS() - m_nEchoClock) > 2000) {
        DoSendEcho(TRUE);
        m_nEchoClock = GetProcesstimeMS();
    }
#else
    if ((clock() - m_nEchoClock) > 2000) {
        DoSendEcho(TRUE);
        m_nEchoClock = clock();
    }
#endif

    return TRUE;
}

BOOL CGXClientLGV::OnEchoBack()
{
    // TODO!

    return TRUE;
}

/**
 *  Sends a echo packet to server.
 *
 *  @param bEchoBack    Echo back; in order to estimate the roundtrip latency.
 */
BOOL CGXClientLGV::DoSendEcho(BOOL bEchoBack)
{
#ifdef LINUX_COMPILE
    #ifndef LOWORD
	#define LOWORD(l) ((unsigned short)((unsigned long)(l) & 0xffff))
    #endif
    #ifndef HIWORD
        #define HIWORD(l) ((unsigned short)((unsigned long)(l) >> 16))
    #endif
#endif // LINUX_COMPILE
#ifdef LINUX_COMPILE
    unsigned long clk = GetProcesstimeMS();
#else
    unsigned long clk = (unsigned long)clock();
#endif
    short data1 = LOWORD(clk);
    short data2 = HIWORD(clk);
//printf("SEND: clk=%lu clkdata1=%hi data2=%hi\n",clk,data1,data2);
    return DoSendInput(0, (bEchoBack) ? (0x80) : (0x00), data1, data2);
}

/**
 *  NEW: Sends a BYE message to server.
 */
BOOL CGXClientLGV::DoSendBye()
{
    m_bConnected = FALSE;
    return DoSendMsg("BYE");
}

/**
 *  NEW: Sends a message to server:
 *  BYE | LOG<anything>
 *
 *  @param szMsg Message.
 */
BOOL CGXClientLGV::DoSendMsg(const char* szMsg)
{
    int len = strlen(szMsg)+1;
    int cs = (int)sizeof(struct sockaddr_in);
	int bytes = sendto(m_sock.m_sd, szMsg, len, 0, (struct sockaddr*)&m_addrServer, cs);
    if (bytes != len) {
#ifdef LINUX_COMPILE
    	GSTRACE("CGXClientLGV::OnConnect(): Failed to send msg\n");
#else
        GSTRACE("CGXClientLGV::DoSendMsg(): Failed to send to %u.%u.%u.%u: %d\n", 
            PRNIP(m_addrServer), WSAGetLastError());
#endif // LINUX_COMPILE
        return FALSE;
	}
    return TRUE;
}

/**
 *  Sends a quit packet to server, without checking for any response.
 *
 *  @DEPRECATED
 */
BOOL CGXClientLGV::DoSendQuit()
{
   if (!m_bConnected)
	return FALSE;

   char* quitpkt = "QUIT";
   int quitlen = strlen(quitpkt);
   int cs = (int)sizeof(struct sockaddr_in);

   int bytes = sendto(m_sock.m_sd, quitpkt, quitlen, 0, (struct sockaddr*)&m_addrServer, cs);

   if (bytes != quitlen)
   {
	GSTRACE("Failed to send QUIT packet");
	return FALSE;
   }

   GSTRACE("Sent QUIT packet");
   return TRUE;
}


#ifndef LINUX_COMPILE
BOOL CGXClientLGV::DoWork()
{
    // Socket?
    if (m_sock.m_sd == (int) NULL)
        return TRUE;

    // Listen
    ULONG nBytes = DoListen();
    if (nBytes == 0)
        return TRUE;

    // On connect
    if (!m_bConnected) {
        OnConnect();
        return TRUE;
    }

    // On vidpkt
    OnVidPkt();

    return TRUE;
}

/**
 *  Sends a user input device packet to server.
 */
BOOL CGXClientLGV::DoSendInputWM(UINT nMsg, WPARAM wParam, LPARAM lParam)
{
    switch (nMsg)
    {
    // Mouse:
    // ------
    // Note: Mouse move must be converted to relative coords first!
    case WM_MOUSEMOVE:
        return DoSendInput(2, 0x00, LOWORD(lParam), HIWORD(lParam));
    case 0x020A /*WM_MOUSEWHEEL*/:
        return DoSendInput(2, 0x01, HIWORD(wParam), 0);
    case WM_LBUTTONDOWN:
        return DoSendInput(2, 0x80, 0x80, 0);
    case WM_LBUTTONUP:
        return DoSendInput(2, 0x80, 0x00, 0);
    case WM_RBUTTONDOWN:
        return DoSendInput(2, 0x81, 0x80, 0);
    case WM_RBUTTONUP:
        return DoSendInput(2, 0x81, 0x00, 0);
    case WM_MBUTTONDOWN:
        return DoSendInput(2, 0x82, 0x80, 0);
    case WM_MBUTTONUP:
        return DoSendInput(2, 0x82, 0x00, 0);
    // Keyboard:
    // ---------
    case WM_KEYDOWN:
        if (lParam & 0x4000)
            return TRUE; // ignore autorepeats!
        return DoSendInput(1, wParam, 0x80, 0);
    case WM_KEYUP:
        return DoSendInput(1, wParam, 0x00, 0);
    }
    return FALSE;
}

#endif // ! LINUX_COMPILE

/////////////////////////////////
//////// DEPRECATED /////////////
/////////////////////////////////
#if 0
/**
 * @function		DoConnectHUDStreamInfoRead()
 * @brief		Reads the reply from the HUD and updates parameters
 *
 * @note		Ignores any errors for now
 * 
 * @returns		-2 if not an OK packet, -1 if wrong parameters, 0 if ok
 */
int CGXClientLGV::DoConnectHUDStreamInfoRead(const char* packet)
{
	if ( strncmp(packet,"OK",2) != 0 )
		return -2;
	
	int ret = sscanf(packet,"%*s\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n%i\n",
			// General
			&m_nPacketSize,
				
			// Video
			&m_nVidCodec,
			&m_nVidBitrate,
			&m_nVidFmtDX,
			&m_nVidWidth,
			&m_nVidHeight,
			&m_nVidFps,
				
			// Audio
			&m_nAudCodec,
			&m_nAudBitrate,
			&m_nAudChannels,
			&m_nAudSampleRate,
			&m_nAudBitsPerSample,
				
			// Other..?
			&m_nInpKBLayout	
	);

	return 0;
}
#endif // 0
