/**
 ****************************************************************************
 *
 *  @file   libgxsys.h
 *  @author Ole-Ivar Holthe
 *  @date   2008-02-28
 *  @brief  Defines the Geelix System Library (libgxsys).
 *  @update 02/04/08: Added ifdefs for Linux compilation (LINUX_COMPILE)
 *
 *  Copyright � Gridmedia Technologies AS 1996-2008. All Rights Reserved.
 *
 ****************************************************************************/

#ifndef LIBGXSYS_H
#define LIBGXSYS_H

#define LINUX_COMPILE
#define LINUX_ONLY

#ifdef LINUX_COMPILE
	#include <inttypes.h>
#ifdef LINUX_ONLY
	#include <malloc.h>
#else // MAC
	#include <malloc/malloc.h>
#endif
	#include <string.h>
	#include <netinet/in.h>
	#include <netdb.h>
	#include <unistd.h>
	#include <stdio.h>
	#include <arpa/inet.h>
	#include <sys/ioctl.h>
	#include <stdlib.h>
	#include <time.h>
	#include <sys/time.h>

	#ifndef TRUE
		#define TRUE	1
	#endif // TRUE
	#ifndef FALSE
		#define FALSE	0
	#endif // FALSE

	#define MAXLONGLONG 0x7fffffffffffffffLL
	#define INVALID_SOCKET	-1


   	typedef uint8_t 		BYTE;
	typedef int64_t 		LONGLONG;
	typedef int 			BOOL;
	typedef unsigned long 		ULONG;
	typedef unsigned int 		UINT;

	typedef char			TCHAR;
	typedef const 			TCHAR *LPCTSTR;

	//typedef uint8_t  		SOCKET;
	typedef int8_t	  		SOCKET;

	typedef unsigned int 		UINT_PTR;
	typedef UINT_PTR 		WPARAM;

	typedef long			LONG_PTR;
	typedef LONG_PTR 		LPARAM;
	
	#include <avcodec.h>
	//#include "libavcodec/avcodec.h"
	//#include "libavformat/avformat.h"
	//#include "libswscale/swscale.h"

	#include "pktcache.h"
	#include "sockudp.h"
	//#include "ffmpeglibrary.h"
	//#include "ffmpegframedecoder.h"


	//#include "clientLGV.h"
	//#include "../common/pktcache.h"
	//#include "../common/sockudp.h"
	//#include "../gxprotocols/clientLGV.h"
#else

	/*
	 ****************************************************************************
	 *  TYPES, CONSTANTS & MACROS
	 ****************************************************************************/

	#define MAKELONGLONG(a, b) \
	    ((LONGLONG) (((ULONG) (a)) | ((LONGLONG) ((ULONG) (b))) << 32))

	#define HILONG(l)   ((ULONG) (((LONGLONG) (l) >> 32) & 0xffffffff))

	#define LOLONG(l)   ((ULONG) (l))

	/** Use UTF-8 charset by default */
	#define ECHAR   char

	/*
	 ****************************************************************************
	 *  INCLUDES
	 ****************************************************************************/

	#include "../win32/system/debug.h"
	#include "../common/pixels.h"
	#include "../common/pktcache.h"
	#include "../common/ptrlist.h"
	#include "../common/string.h"
	#include "../common/sockudp.h"
	#include "../common/wrap_jpeg.h"
	#include "../common/wrap_png.h"
	#include "../tests/testcase.h"
	#include "../utility/sproket.h"
	#include "../utility/http.h"
	#include "../win32/system/dib.h"
	#include "../win32/system/internet.h"
	#include "../win32/system/pdh.h"
	#include "../win32/system/wrap_gdiplus.h"
	#include "../gxffmpeg/ffmpeglibrary.h"
	#include "../gxffmpeg/ffmpegframedecoder.h"
	#include "../gxffmpeg/ffmpegframeencoder.h"
	#include "../gxffmpeg/ffmpegdecoder.h"
	#include "../gxffmpeg/scaler.h"
	#include "../gxprotocols/chat.h"
	#include "../gxprotocols/clientLGV.h"
	#include "../gxprotocols/shareLGV50.h"

	#ifdef _TEST
	#include "../tests/test_string.h"
	#include "../tests/test_internet.h"
	#include "../tests/test_gdiplus.h"
	#include "../tests/test_pdh.h"
	#endif // _TEST
#endif // LINUX_COMPILE


#endif // LIBGXSYS_H

/*
 ***************************************************************************/
