/**
 * @file		LiveGamesApp.h
 * @author 		Ola Mogstad (olamo@stud.ntnu.no)
 * @project		Geelix LiveGames Linux
 * @updated		07/03/08
 * @brief		Main header file. Contains the main function.
 * @TODO		Memory allocation and memory leak prevention
 * @TODO		Validate .xml file
 * @TODO		Improve code documentation
 */

#ifndef LIVEGAMESAPP_H
#define LIVEGAMESAPP_H

class GXLinuxClient;

class LiveGamesApp
{
public:
	// constructors & destructors
	LiveGamesApp(const char* filename);
    ~LiveGamesApp();
    
    // the main function
	bool Timeout(int timer_number);
    
    // setters and getters
    Gtk::Window& get_window() const { return *mainWindow; }
    struct UserInfo getUserInfo() const { return userInfo; }
    Glib::RefPtr<Gnome::Glade::Xml> getGladeXML() const { return refXml; }
    void GUI_PrintStatusbar(const char* msg);
    int GetMainLoopRunning() const { return mainLoopRunning; }
    void GUI_CloseDialog() { aboutDialog->hide(); printf("hello"); }
    
	// classes
    class ModelColumns : public Gtk::TreeModel::ColumnRecord
    {
    public:

      ModelColumns()
      { add(m_text); }

      Gtk::TreeModelColumn<Glib::ustring> m_text;
    };
	
private:
	// data members
	Glib::RefPtr<Gnome::Glade::Xml> refXml;
	Gtk::Window* mainWindow;
	Gtk::Dialog* inputDialog;
	Gtk::Dialog* msgDialog;
	Gtk::AboutDialog* aboutDialog;
	Gtk::Statusbar* statusbar;
	Gtk::Button* connectButton;
	struct UserInfo userInfo;
	GXLinuxClient client;
	int mainLoopRunning;
	
	struct ServerInfo* serverList;
	int nServers;	
	struct GameInfo* gameList;
	int nGames;
	char** profiles;
	int nProfiles;
	char** sizes;
	int nSizes;
	char** abitrates;
	int naBitrates;

	// functions
	int GUI_RetrieveUserInput();
	void GUI_BtnConnectClicked();
	void GUI_BtnPingClicked();
	void GUI_BtnRefreshClicked();
	void ToggleDeveloper();
	void ToggleRenderer();
	void SetTimer();
	void SetTimerClick();
	attribute_deprecated void TestInput();
	int RetrieveServerGameData(char* serverChar, char* gamesChar);
	int ParseServerGameData(char* serverChar, char* gamesChar);
	int UpdateLists();
	void PrintServerList();
	void PrintGameList();
	void DefaultLists();
};

int main(int argc, char* argv[])
{
	LOG.MLog("SYSINFO_LINUX\n%s",SYSINFO_LINUX());
	
	if (argc == 2)
	{
		/*if (strncmp(argv[1],"-d",2) == 0)
		{
			printf("\n*** Application running in DEVELOPER mode ***\n\n");
			DEVELOPER = 1;
		}*/
		if ( !strncmp(argv[1],"-version",8) || !strncmp(argv[1],"-v",2) )
		{
			printf("Title: \t\t%s\n",PACKAGE_NAME);
			printf("Version: \t%s\n",PACKAGE_VERSION);
			printf("ffmpeg: \t%s\n",FFMPEG_REVISION);
			printf("Author: \t%s\n",PACKAGE_BUGREPORT);
			return 0;
		}
		else
		{
			printf("Possible triggers:\n");
			printf("-v: Show version information\n");
			return 0;
		}
	}
	
    try
    {
        Gtk::Main kit(argc, argv);
        LiveGamesApp livegames(GLADE_FILE);
        kit.run(livegames.get_window());
        return 0;
    }
    catch (std::exception const& ex)
    {
        std::cerr << "*** ERROR: " << ex.what() << " ***" << std::endl;
        printf("Is there version mismatch between source code and GUI XML file..?\n");
        return 1;
    }
	return 0;
}

#endif // LIVEGAMESAPP_H
