/**
 * @file		StopWatch.h
 * @author 		Ola Mogstad (olamo@stud.ntnu.no)
 * @updated		27/03/08
 * 
 * @class		StopWatch
 * @brief		Implements a stopwatch with lap functions
 * 
 * @TODO		Determine real accuracy of this
 */

#ifndef STOPWATCH_H
#define STOPWATCH_H

#include <stdlib.h>
#include <sys/time.h>
#include <time.h>

static int MAX_INTERVALS = 100000;

class StopWatch
{
public:
			// member functions
			StopWatch();
			~StopWatch();
			long LapStart();
			long LapStop();
			void Reset() { running = 0; intervals = 0; totalTime = 0; maxLap = 0; minLap = 100000; }
			void Print() { printf("%li us over %i intervals (max=%li min=%li)\n",totalTime,intervals,maxLap,minLap); }
			void PrintData() { printf("%s\n",dataStr); }

			
private:
			// member functions
			long GetTimeDif(struct timeval start, struct timeval finish);
			
			// data members
			struct timeval start,finish;
			int running;
			
			// statistics
			int intervals;
			long totalTime;
			long maxLap;
			long minLap;
			
			// performance tests
			char* dataStr;
};

StopWatch::StopWatch()
{
	Reset(); 
	dataStr = (char*) calloc(MAX_INTERVALS*10, sizeof(char));
}

StopWatch::~StopWatch()
{
	free(dataStr);
}

/**
 * @function	LapStart()
 * @brief		Starts a lap
 * 
 * @returns 	-1 if already running, total time so far if OK
 */
long StopWatch::LapStart()
{ 
	if (running == 1)
		return -1;
	
	gettimeofday(&start,NULL); 
	running = 1; 
	return totalTime; 
}

/**
 * @function	LapStop()
 * @brief		Stops a lap
 * 
 * @returns 	-1 if not running, total time so far if OK
 */
long StopWatch::LapStop()
{ 
	if (running == 0)
		return -1;
	
	gettimeofday(&finish,NULL); 
	
	long tmpTime = GetTimeDif(start,finish);
	
	if (tmpTime > maxLap)
		maxLap = tmpTime;
	if (tmpTime < minLap)
		minLap = tmpTime;
	
	sprintf(dataStr,"%s%li\n",dataStr,tmpTime);
	
	totalTime += tmpTime;
	running = 0;
	intervals++;
	return totalTime;
}

/**
 * @function	GetTimeDif()
 * @brief		Returns time difference between start and finish in microseconds
 */ 
long StopWatch::GetTimeDif(struct timeval start, struct timeval finish) 
{ 
	return (finish.tv_sec - start.tv_sec)*1000000 + (finish.tv_usec - start.tv_usec);
}

#endif // STOPWATCH_H
