/**
 * @file		Sysinfo.h
 * @author 		Ola Mogstad (olamo@stud.ntnu.no)
 * @updated		28/04/08 
 * @brief		Defines function(s) for retrieving system information on a computer
 */

#ifndef SYSINFO_H
#define SYSINFO_H

using namespace std;

#ifndef LINUX_COMPILE
	// TODO
#else

/* Note: Since linux system files in /proc/ folder has zero size, it is 
 * not possible to use file.tellg() to determine their size. Must handle
 * this by simply overallocating memory space, assuming that the files 
 * are not extraordinary huge. */
static int MAX_FILE_SIZE = 1024*10;
#define SYSINFO_LINUX_CPUINFO 	"/proc/cpuinfo"
#define SYSINFO_LINUX_DISTRO 	"/etc/issue"
#define SYSINFO_LINUX_VERSION 	"/proc/version"
char* output;

char* SYSINFO_LINUX();
	
char* SYSINFO_LINUX()
{
	char* cpuinfo = new char[MAX_FILE_SIZE];
	char* distro = new char[MAX_FILE_SIZE];
	char* version = new char[MAX_FILE_SIZE];
#ifdef LINUX_ONLY
	ifstream file;
	// CPU INFO
	file.open(SYSINFO_LINUX_CPUINFO, ios::in);
	if (file.is_open())
	{
		file.read(cpuinfo, MAX_FILE_SIZE);
		//printf("cpuinfo: %s",cpuinfo);
		file.close();
	}
	else
		strcpy(cpuinfo,"ERROR: FAILED TO OPEN SYSTEM FILE");
	
	// DISTRO
	file.open(SYSINFO_LINUX_DISTRO, ios::in);
	if (file.is_open())
	{
		file.read(distro, MAX_FILE_SIZE);
		//printf("distro: %s",distro);
		file.close();
	}
	else
		strcpy(distro,"ERROR: FAILED TO OPEN SYSTEM FILE");
	
	// VERSION
	file.open(SYSINFO_LINUX_VERSION, ios::in);
	if (file.is_open())
	{
		file.read(version, MAX_FILE_SIZE);
		//printf("version: %s",version);
		file.close();
	}
	else
		strcpy(version,"ERROR: FAILED TO OPEN SYSTEM FILE");
#else // MAC
	strcpy(cpuinfo,"N/A");
	strcpy(distro,"Mac OS X");
	strcpy(version,"N/A");
#endif
	
	output = new char[MAX_FILE_SIZE*3 + 128];
	bzero(output,sizeof(output));
	sprintf(output,"*** CPUINFO ***\n%s\n*** DISTRO ***\n%s\n*** VERSION ***\n%s\n",
			cpuinfo, distro, version);
	
	delete [] cpuinfo;
	delete [] distro;
	delete [] version;
	
	return output;
}

#endif // ! LINUX_COMPILE

#endif // SYSINFO_H
