/**
 * @file		defines.h
 * @author 		Ola Mogstad (olamo@stud.ntnu.no)
 * @brief		Makes all #defines needed, used by all other classes
 */

/**
 * @global	LINUX_COMPILE
 * @brief	Contains stuff that is to be included for linux and mac
 * @global	LINUX_ONLY
 * @brief	Contains stuff that is to be included for linux only, and not mac
 */
#define LINUX_COMPILE
//#define LINUX_ONLY

/**
 * @global	USEGXSYS
 * @brief	If defined, compilation will use code from libgxsys
 * @note	MUST BE DEFINED
 */
#define USEGXSYS

/**
 * @global	NEW_FFMPEG / OLD_FFMPEG
 * @brief	Defines what version of ffmpeg to use. Note that if changed, changes
 * 				must also be made in file structure and configure.ac
 * @note	libswscale is GPL and not LGPL
 */
//#define NEW_FFMPEG
#define OLD_FFMPEG

#ifdef NEW_FFMPEG
	#define USESWSCALE
	#define FFMPEG_REVISION "12893"
#else
	#define FFMPEG_REVISION "8397"
#endif

/**
 * @global	DEVELOPER <0/1>
 * @brief	Sets if application should compile in developer mode or not (lots of printouts)
 * @note	Can be toggled in GUI
 * @default	0
 */
const int DEVELOPER = 0;

/**
 * @global PERFORMANCETEST
 * @brief If defined, performance tests will be carried out while executing
 * @default undefined
 */
//#define PERFORMANCETEST

/**
 * @global	GTK_TIMER <int>
 * @brief	Sets the timer used for the GTK application
 * @note	For high values: Much of the input is queued and delayed
 * @default	5 (200 Hz)
 */
static int GTK_TIMER = 5;

/**
 * @global	LIMITINPUTPUSH <0/x>
 * @brief	If >0, limits the amount of input events that are handled on each DoWork() iteration.
 * @default 10
 */
static int LIMITINPUTPUSH = 10;

/**
 * @global	RENDER_VIDEO <0/1>
 * @global	RENDER_AUDIO <0/1>
 */
static int RENDER_VIDEO = 1;
static int RENDER_AUDIO = 1;

/**
 * @define	AUDIO_RENDER_BUFFER
 * @brief	Sets the audio buffer for the SDL rendering in samples. Higher value is more 
 * 				value but decreases	responsiveness. Recommended value in range 512-8192.
 * @define	SUPERBUFFER_SIZE
 * @brief	Sets maximum size of super buffer. Data beyond this amount will be ignored.
 */
static int AUDIO_RENDER_BUFFER = 1024*8;
static int SUPERBUFFER_SIZE = 8192*8;

/**
 * @define	AUDIO_DECODER_HOLDBACK
 * @brief	Sets the amount of samples that the audio decoder will hold back before it 
 * 				returns the whole thing. Set to 0 to disable.
 */
static int AUDIO_DECODER_HOLDBACK = 0;

/**
 * @define	URLs
 * @brief	URLs to web pages that retrieve data
 */
#define URL_GETGAMES 	"http://www.geelix.com/services/livegames.5.0.0.0/getgames.php"
#define URL_GETSERVERS	"http://www.geelix.com/services/livegames.5.0.0.0/getservers.php"
#define MAX_URL_DATA	4096	// Max data content of .php script results 

/**
 * @define	GUI
 * @brief	Different configurations for the GUI
 */
#define GLADE_FILE 				"/usr/include/livegames/gui/LiveGames.glade"		// Absolute path
#define FONT_VERA12				"/usr/include/livegames/gui/Vera.ttf"				// Absolute path
#define FONT_BITMAP				"/usr/include/livegames/gui/24P_Arial_NeonYellow.png"				
#define USER_INPUT_SIZE 			256
#define DEFAULT_CONNECT_PORT 		13578
#define DEFAULT_DATA_PORT 			13580
#define MAX_CONTROL_PACKET		 	512
#define MAX_INPUT_PACKET 			300
#define MAX_DATA_PACKET 			8192
#define MAX_FRAME_SIZE 				4194304 	// 4MB
#define HEADER_SIZE 				21

#define FREQ_COUNT 					20			// Number of events to use for input stats

enum State
{
	IDLE,		// Disconnected: Not in any communications
	WAIT_LS,	// Waiting for response from LiveStart
	WAIT_HUD,	// Waiting for response from HUD
	CONNECTED,	// Connected to a HUD
	FILEPLAY	// Streaming from a file
};

static enum State state;

// TYPEDEFS
typedef int BOOL;

////////////////////////////////////////
//////  DEPRECATED /////////////////////
////////////////////////////////////////

/**
 * @define	SDLRENDER <0/1>
 * @brief	Render decoded frames on SDL window
 * @default	1
 */
//static int SDLRENDER = 1;

/**
 * @define	SAVEFRAMEPPM
 * @brief	Instructs the renderer to save frames as .ppm files
 * @default undefined
 */
//#define SAVEFRAMEPPM

/**
 * @define	SDLINPUTFREQ
 * @brief	Toggle calculation of the SDL input frequency (cpu intensive)
 * @note	DEPRECATED
 */
//#define SDLINPUTFREQ


/**
 * @global	LIMITFRAMESDEC <0/x>
 * @brief	If >0, quits the client after having decoded <x> frames
 * @note	DEPRECATED
 */
//static int LIMITFRAMESDEC = 0;

/**
 * @global	USEFFMPEGLIB
 * @brief	Toggles use of old ffmpeg_encoder_decoder library
 * 
 * @note	DEPRECATED
 */
//#define USEFFMPEGLIB
