/**
 ****************************************************************************
 *
 *  @file   clientLGV.h
 *  @author Ole-Ivar Holthe
 *  @date   2008-04-01
 *  @brief  Defines the CGXClientLGV class.
 *
 *  Copyright  Gridmedia Technologies AS 1996-2008. All Rights Reserved.
 *
 ****************************************************************************/

#ifndef GX_CLIENTLGV_H
#define GX_CLIENTLGV_H


#include "libgxsys.h"

#ifndef LINUX_COMPILE
	#include "internal.h"
#endif // LINUX_COMPILE

class CGXClientLGV
{
public:
    CGXClientLGV();
    virtual ~CGXClientLGV();

// Protocol functions
public:
    virtual BOOL  DoSendConnectVideo(const char* szServerIP, int nServerPort = 13580);
    virtual ULONG DoListen();
    virtual BOOL  OnConnect();
    virtual BOOL  DoSendInput(short nDevice, short nVirtKey, short nData1, short nData2);
    virtual BOOL  OnVidPkt();
    virtual BOOL  OnEchoBack();
    virtual BOOL  DoSendEcho(BOOL bEchoBack);
    virtual BOOL  DoSendBye();
    virtual BOOL  DoSendMsg(const char* szMsg);

    attribute_deprecated virtual BOOL  DoSendQuit();

#ifndef LINUX_COMPILE
    virtual BOOL  DoWork();
    virtual BOOL  DoSendInputWM(UINT nMsg, WPARAM wParam, LPARAM lParam);
#endif // LINUX_COMPILE

// Protocol members
public:
    char m_szLoginName[1024];
    char m_szLoginPassword[1024];
    int  m_nLoginGameID;
#ifdef LINUX_COMPILE
    unsigned long m_nEchoClock;
    unsigned long m_nEchoLatency;
#else
    clock_t m_nEchoClock;
    clock_t m_nEchoLatency;
#endif
    UINT m_nSendInputSeqNo;

// Protocol attributes
public:
    int  m_nPacketSize;
    int  m_nVidCodec;
    int  m_nVidBitrate;
    int  m_nVidFmtDX;
    int  m_nVidWidth;
    int  m_nVidHeight;
    int  m_nVidFps;
    int  m_nAudCodec;
    int  m_nAudBitrate;
    int  m_nAudChannels;
    int  m_nAudSampleRate;
    int  m_nAudBitsPerSample;
    int  m_nInpKBLayout;

// Members
public:
    BOOL m_bConnected;
    CGSSockUDP m_sock;
    CGSPktCache m_cache;
    struct sockaddr_in m_addrServer;
    BYTE m_pktbuf[65536]; // 64k

#ifdef LINUX_COMPILE
	struct timeval start;
	unsigned long GetProcesstimeMS()
	{
		timeval now;
		gettimeofday(&now,NULL);
		return ( (now.tv_sec - start.tv_sec)*1000000 + (now.tv_usec - start.tv_usec) / 1000 );
	} 
#endif
};


#endif // GX_CLIENTLGV_H

/*
 ***************************************************************************/
