
#include <Carbon/Carbon.h>
#include "TApplication.h"
#include "TWindow.h"

#include "globals.h"
#include "includes.h"
#include "structs.h"
#include "GXLinuxRenderer.h"
#include "GXLinuxDecoder.h"
#include "GXLinuxInputHandler.h"
#include "GXLinuxClient.h"

GXLinuxClient client;

// Our custom application class
class CarbonApp : public TApplication
{
    public:
							
                            CarbonApp() {}
        virtual             ~CarbonApp() {}
        
    protected:
        virtual Boolean     HandleCommand( const HICommandExtended& inCommand );
};

// Our main window class
class MainWindow : public TWindow
{
    public:
                            MainWindow() : TWindow( CFSTR("MainWindow") ) {}
        virtual             ~MainWindow() {}
        
        static void         Create();
        
    protected:
		static pascal void	HandleTimer(
									EventLoopTimerRef			inTimer,
									void*						inUserData );
        virtual Boolean     HandleCommand( const HICommandExtended& inCommand );
        virtual Boolean     HandleConnect( const HICommandExtended& inCommand );
		
		virtual Boolean     GetControlEditText(char* pszTarget, int nMax, int nCtrlID);
		virtual int         GetControlEditValue(int nCtrlID);
		virtual int         GetControlCheckValue(int nCtrlID);

};

//--------------------------------------------------------------------------------------------
int main(int argc, char* argv[])
{
    CarbonApp app;
    
    // Create a new window. A full-fledged application would do this from an AppleEvent handler
    // for kAEOpenApplication.
    MainWindow::Create();
    
    app.Run();
    return 0;
}

//--------------------------------------------------------------------------------------------
Boolean
CarbonApp::HandleCommand( const HICommandExtended& inCommand )
{
    switch ( inCommand.commandID )
    {
        case kHICommandNew:
            MainWindow::Create();
            return true;
			     
		// Quit button
        case 202:
            QuitApplicationEventLoop();
            return true;
			
        // Add your own command-handling cases here
        
        default:
            return false;
    }
}

//--------------------------------------------------------------------------------------------
void
MainWindow::Create()
{
    MainWindow* wind = new MainWindow();

    // Position new windows in a staggered arrangement on the main screen
    RepositionWindow( *wind, NULL, kWindowCascadeOnMainScreen );
    
    // The window was created hidden, so show it
    wind->Show();
	
	// Install a timer
	/*OSStatus err;
	EventLoopTimerRef fTimer;
	err = InstallEventLoopTimer( GetCurrentEventLoop(), 0.01, 0.01,
			HandleTimer, wind, &fTimer );*/
}



//-----------------------------------------------------------------------------------
pascal void
MainWindow::HandleTimer(
	EventLoopTimerRef			inTimer,
	void*						inUserData )
{
	#pragma unused( inTimer )
	
	// Handles receiving video/audio and sending input
	//client.DoWork();
	
	//HIViewSetNeedsDisplay( (HIViewRef) inUserData, true );
}

//--------------------------------------------------------------------------------------------
Boolean
MainWindow::HandleCommand( const HICommandExtended& inCommand )
{
    switch ( inCommand.commandID )
    {
		// Connect button
		case 200:
		    return HandleConnect(inCommand);
						
		// Refresh button
		case 201:
		    return true;
        
        default:
            return false;
    }
}

//--------------------------------------------------------------------------------------------
Boolean
MainWindow::HandleConnect( const HICommandExtended& inCommand )
{
	// Retrieve user info from GUI
	UserInfo userInfo;
	userInfo.Clear();
	GetControlEditText(userInfo.username, USER_INPUT_SIZE, 101);
	GetControlEditText(userInfo.password, USER_INPUT_SIZE, 102);
	userInfo.fullscreen = GetControlCheckValue(103);
	userInfo.grabinput = GetControlCheckValue(104);
	//userInfo.ingamecursor = GetControlCheckValue(105);
	strcpy(userInfo.game,"(Running game)");
	GetControlEditText(userInfo.profile, USER_INPUT_SIZE, 106); // 768 Kbps
	GetControlEditText(userInfo.abitrate, USER_INPUT_SIZE, 107); // 64 Kbps
	GetControlEditText(userInfo.size, USER_INPUT_SIZE, 109); // Normal (1:1)
	userInfo.packetsize = GetControlEditValue(108);
	userInfo.framerate = GetControlEditValue(110);
	userInfo.infoRetrieved = 1;
	userInfo.port = GetControlEditValue(111);
	
	char tmpstr[100];
	GetControlEditText(tmpstr, USER_INPUT_SIZE, 100);
	if ( Netstuff::SplitServerPort(tmpstr,userInfo.server,&userInfo.port) == 0 )
		userInfo.port = 80;
	
	// Initiate the client with retrieved data
	client.InitClient(userInfo);
	
	// Connect to the server
	client.DoConnect();
	
	// HACK: This operation will freeze the GUI until SDL window quits
	while(state != IDLE)
	{
		usleep(5*1000);
		client.DoWork();
	}
	
	return true;
}

//--------------------------------------------------------------------------------------------
Boolean
MainWindow::GetControlEditText(char* pszTarget, int nMax, int nCtrlID)
{
	OSStatus err;
    ControlRef cmbCtrl;
	Size size;

	// Get form value
	ControlID cmbServerID = { 'TEXT', nCtrlID };
	err = GetControlByID( GetWindowRef(), &cmbServerID, &cmbCtrl );
	if (err != 0) return false;
	
	::GetControlData( cmbCtrl, 0, kControlEditTextTextTag, 0, NULL, &size );
	if ( size >= nMax ) size = nMax-1;
	::GetControlData( cmbCtrl, 0, kControlEditTextTextTag, size, pszTarget, NULL );
	pszTarget[size] = 0;
	
	return true;
}

//--------------------------------------------------------------------------------------------
int
MainWindow::GetControlEditValue(int nCtrlID)
{
	OSStatus err;
    ControlRef cmbCtrl;
	Size size;
    char text[255];

	// Get form value
	ControlID cmbServerID = { 'TEXT', nCtrlID };
	err = GetControlByID( GetWindowRef(), &cmbServerID, &cmbCtrl );
	if (err != 0) return 0;
	
	::GetControlData( cmbCtrl, 0, kControlEditTextTextTag, 0, NULL, &size );
	if ( size >= 255 ) size = 254;
	::GetControlData( cmbCtrl, 0, kControlEditTextTextTag, size, text, NULL );
	text[size] = 0;
	
	return atoi( (const char*)text );
}

//--------------------------------------------------------------------------------------------
int
MainWindow::GetControlCheckValue(int nCtrlID)
{
	OSStatus err;
    ControlRef cmbCtrl;
	
	// Get form value
	ControlID cmbServerID = { 'CHCK', nCtrlID };
	err = GetControlByID( GetWindowRef(), &cmbServerID, &cmbCtrl );
	if (err != 0) return 0;
		
	return GetControlValue(cmbCtrl);
}
