/**
 * @file		StopWatch.h
 * @author 		Ola Mogstad (olamo@stud.ntnu.no)
 * @updated		27/03/08
 * 
 * @class		StopWatch
 * @brief		Implements a stopwatch with lap functions
 * 
 * @TODO		Determine real accuracy of this
 */

#ifndef STOPWATCH_H
#define STOPWATCH_H

#include <sys/time.h>
#include <time.h>

class StopWatch
{
public:
			// member functions
			StopWatch() { running = 0; intervals = 0; }
			long LapStart();
			long LapStop();
			long GetTotalTime() { return totalTime; }
			long GetIntervals() { return intervals; }
			
private:
			// member functions
			long GetTimeDif(struct timeval start, struct timeval finish);
			
			// data members
			struct timeval start,finish;
			long totalTime;
			int running;
			int intervals;
};

/**
 * @function	LapStart()
 * @brief		Starts a lap
 * 
 * @returns 	-1 if already running, total time so far if OK
 */
long StopWatch::LapStart()
{ 
	if (running == 1)
		return -1;
	
	gettimeofday(&start,NULL); 
	running = 1; 
	return totalTime; 
}

/**
 * @function	LapStop()
 * @brief		Stops a lap
 * 
 * @returns 	-1 if not running, total time so far if OK
 */
long StopWatch::LapStop()
{ 
	if (running == 0)
		return -1;
	
	gettimeofday(&finish,NULL);  
	totalTime += GetTimeDif(start,finish);
	running = 0;
	intervals++;
	return totalTime;
}

/**
 * @function	GetTimeDif()
 * @brief		Returns time difference between start and finish in microseconds
 */ 
long StopWatch::GetTimeDif(struct timeval start, struct timeval finish) 
{ 
	return (finish.tv_sec - start.tv_sec)*1000000 + (finish.tv_usec - start.tv_usec);
}

#endif // STOPWATCH_H
